--[[
	Copyright (C) Udorn (Blackhand)
--]]

lootlog = lootlog or {}

lootlog.LootLog = LibStub("AceAddon-3.0"):NewAddon("Lootlog", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local L = lootlog.Locale.GetInstance()
local log = lootlog.Debug:new("Lootlog")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LibDeformat = LibStub("LibDeformat-3.0")

local APP_NAME = "Lootlog"

local order = 1

local SCREEN_HEIGHT = GetScreenHeight()
local SCREEN_WIDTH = GetScreenWidth()

local PLAYER = 1
local ITEM = 2
local DICE = 3

local function _NextOrder()
	order = order + 1
	return order
end

local DEFAULT_CONFIG = {
	profile = {
		backgroundColor = {0.282, 0.239, 0.541, 0.62},
		lostBackgroundColor = {0.976, 0.114, 0.112, 1},
		lostNeedOnly = true,
		wonBackgroundColor = {0.31, 0.91, 0.216, 1},
		wonNeedOnly = true,
		fadeoutTime = 30,
		showClass = false,
		colorizePlayerName = true,
		scale = 1,
		minQuality = 2,
		activeGroup = true,
		activeRaid10 = true,
		activeRaid25 = true,
		activeRaid40 = true,
		windowPosition = {
			point = "TOPLEFT",
			relativePoint = "TOPLEFT", 
			x = SCREEN_WIDTH / 15, 
			y = -(SCREEN_HEIGHT / 3)
		}
	}
}

local OPTIONS = {
	--name = APP_NAME,
	handler = lootlog.LootLog,
	type = 'group',
	childGroups = "tab",
	args = {
		locked = {
			type = 'toggle',
			width = "half",
			name = L["Locked"],
			desc = L["Locks the window"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.locked = value; end,
			get = function(info) return info.handler.db.profile.locked end,
		},
		test = {
			type = 'toggle',
			name = L["Test"],
			desc = L["Activates the test mode for moving the window and modifying the layout"],
			order = _NextOrder(),
			set = function(info, value) info.handler.db.profile.testMode = value; info.handler.lootWindow:TestModeUpdate() end,
			get = function(info) return info.handler.db.profile.testMode end,
		},
	 	main = {
		 	type="group",
		 	name = L["View"],
		 	order = _NextOrder(),
		 	args = {
				fadeoutTime = {
					type = 'range',
					name = L["Fadeout time"],
					desc = L["Number of seconds an item is still visible after it has been won"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.fadeoutTime = value end,
					get = function(info) return info.handler.db.profile.fadeoutTime end,
					min = 1,
					max = 60,
					step = 1,
				},
				scale = {
					type = 'range',
					name = L["Scale"],
					desc = L["Scale factor of the Lootlog window"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.scale = value end,
					get = function(info) return info.handler.db.profile.scale end,
					min = 0.1,
					max = 3,
					step = 0.01,
				},
				newLine1 = {
					type = 'description',
					name = "\n",
					order = _NextOrder(),
				},
				showClass = {
					type = 'toggle',
					name = L["Show class"],
					desc = L["Adds the class to the player name"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.showClass = value end,
					get = function(info) return info.handler.db.profile.showClass end,
				},
				colorizePlayerName = {
					type = 'toggle',
					name = L["Class colorization"],
					desc = L["Colorize the player names by their class"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.colorizePlayerName = value end,
					get = function(info) return info.handler.db.profile.colorizePlayerName end,
				},
				newLine2 = {
					type = 'description',
					name = "\n",
					order = _NextOrder(),
				},
				minQuality = {
					type = "select",
					name = L["Minimum quality"],
					desc = L["Only show rolls for items with at least the selected quality"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.minQuality = value; info.handler:Update() end,
					get = function(info) return info.handler.db.profile.minQuality end,
					values = {[0] = ITEM_QUALITY0_DESC, [1] = ITEM_QUALITY1_DESC, [2] = ITEM_QUALITY2_DESC, [3] = ITEM_QUALITY3_DESC, [4] = ITEM_QUALITY4_DESC, [5] = ITEM_QUALITY5_DESC}
				},
				newLine3 = {
					type = 'description',
					name = "\n",
					order = _NextOrder(),
				},
				wonBackgroundColor = {
					type = 'color',
					name = L["Win Background"],
					desc = L["Background color for rolls won"],
					order = _NextOrder(),
					set = function(info, ...) info.handler.db.profile.wonBackgroundColor = {...}; end,
					get = function(info) return unpack(info.handler.db.profile.wonBackgroundColor) end,
					hasAlpha = true
				},
				wonNeedOnly = {
					type = 'toggle',
					name = L["Need only"],
					desc = L["Uses the given background color only for need rolls"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.wonNeedOnly = value end,
					get = function(info) return info.handler.db.profile.wonNeedOnly end,
				},
				lostNewLine = {
					type = 'description',
					name = "",
					order = _NextOrder(),
				},
				lostBackgroundColor = {
					type = 'color',
					name = L["Loss Background"],
					desc = L["Background color for rolls lost"],
					order = _NextOrder(),
					set = function(info, ...) info.handler.db.profile.lostBackgroundColor = {...}; end,
					get = function(info) return unpack(info.handler.db.profile.lostBackgroundColor) end,
					hasAlpha = true
				},
				lostNeedOnly = {
					type = 'toggle',
					name = L["Need only"],
					desc = L["Uses the given background color only for need rolls"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.lostNeedOnly = value end,
					get = function(info) return info.handler.db.profile.lostNeedOnly end,
				},
				backgroundColorNewLine = {
					type = 'description',
					name = "",
					order = _NextOrder(),
				},
				backgroundColor = {
					type = 'color',
					name = L["Background"],
					desc = L["General background color"],
					order = _NextOrder(),
					set = function(info, ...) info.handler.db.profile.backgroundColor = {...}; end,
					get = function(info) return unpack(info.handler.db.profile.backgroundColor) end,
					hasAlpha = true
				},
				activeDesc = {
					type = 'header',
					name = L["Activation"],
					desc = L["Selects the instance types for using Lootlog"],
					order = _NextOrder(),
				},
				activeGroup = {
					type = 'toggle',
					name = L["Group"],
					desc = L["Uses Lootlog in groups"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.activeGroup = value end,
					get = function(info) return info.handler.db.profile.activeGroup end,
				},
				activeRaid10 = {
					type = 'toggle',
					name = L["Raid 10"],
					desc = L["Uses Lootlog in raids of 10 people"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.activeRaid10 = value end,
					get = function(info) return info.handler.db.profile.activeRaid10 end,
				},
				activeRaid25 = {
					type = 'toggle',
					name = L["Raid 25"],
					desc = L["Uses Lootlog in raids of 25 people"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.activeRaid25 = value end,
					get = function(info) return info.handler.db.profile.activeRaid25 end,
				},
				activeRaid40 = {
					type = 'toggle',
					name = L["Raid 40"],
					desc = L["Uses Lootlog in raids of 40 people"],
					order = _NextOrder(),
					set = function(info, value) info.handler.db.profile.activeRaid40 = value end,
					get = function(info) return info.handler.db.profile.activeRaid40 end,
				},
			}
		}
	}
}

local function _IsActive(self)
	local name, type, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo()
	local rtn = false
	if (self.db.profile.activeParty and type == "party") then
		rtn = true
	elseif (type == "raid") then
		rtn = (self.db.profile.activeRaid10 and maxPlayers <= 10) or 
			(self.db.profile.activeRaid25 and maxPlayers > 10 and maxPlayers <= 25) or 
			(self.db.profile.activeRaid40 and maxPlayers > 25)
	end
	log:Debug("IsActive rtn [%s] name [%s] type [%s] difficultyIndex [%s] difficultyName [%s] maxPlayers [%s] dynamicDifficulty [%s] isDynamic [%s]", rtn, name, type, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic)
	return rtn
end

local function _StripItemName(name)
	return GetItemInfo(name)
end

local function _StripDice(dice)
	-- too lazy to fix the regexp
	local s = string.gsub(dice, "%-", "")
	return strtrim(s)
end

function lootlog.LootLog:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LootLogDb", DEFAULT_CONFIG, "Default")
	OPTIONS.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.db.profile.testMode = nil -- just in case
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileUpdate")
end

function lootlog.LootLog:OnEnable()
	self.lootWindow = lootlog.LootWindow:new()
	self.playerName = GetUnitName("player")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable(APP_NAME, OPTIONS)
	self.configFrame = AceConfigDialog:AddToBlizOptions(APP_NAME)
	self:RegisterChatCommand("lootlog", "ChatCommand")
		
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("START_LOOT_ROLL")
--	self:RegisterEvent("CANCEL_LOOT_ROLL")
--	self:RegisterEvent("CONFIRM_DISENCHANT_ROLL")
--	self:RegisterEvent("CONFIRM_LOOT_ROLL")

	-- LOOT_ROLL_ROLLED_NEED_ROLE_BONUS = "Need Roll - %d for %s by %s + Role Bonus";
--	local text = "Wurf für Bedarf: 197 für |cffa335ee|Hitem:78485:0:0:0:0:0:0:432663696:85:0|h[Schlund des Drachenlords]|h|r von Alrock + Rollenbonus"
--	local pattern = LOOT_ROLL_ROLLED_NEED_ROLE_BONUS
--	local dice, itemName, playerName = LibDeformat.Deformat(text, pattern)
--	log:Debug("dice [%s] itemName [%s] playerName [%s]", dice, itemName, playerName)
end

function lootlog.LootLog:CHAT_MSG_LOOT(evt, msg, id)
	log:Debug("CHAT_MSG_LOOT msg [%s] id [%s]", msg, id)

	if (not msg) then
		return
	end
	if (not _IsActive(self)) then
		return
	end
	
	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_NEED_SELF)
	if (itemName) then
		log:Debug("1")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.NEED, _StripItemName(itemName))
		return
	end
	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_GREED_SELF)
	if (itemName) then
		log:Debug("2")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.GREED, _StripItemName(itemName))
		return
	end
	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_DISENCHANT_SELF)
	if (itemName) then
		log:Debug("3")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.DISENCHANT, _StripItemName(itemName))
		return
	end
	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_PASSED_SELF)
	if (itemName) then
		log:Debug("4")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.PASSED, _StripItemName(itemName))
		return
	end
	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_PASSED_SELF_AUTO)
	if (itemName) then
		log:Debug("5")
		self.lootWindow:SetPlayerRoll(lootlog.LootWindow.PASSED, _StripItemName(itemName))
		return
	end

	local dice, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_YOU_WON_NO_SPAM_NEED)
	if (match1) then
		log:Debug("6")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, dice, _StripItemName(itemName), self.playerName)
		return
	end
	local dice, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_YOU_WON_NO_SPAM_GREED)
	if (match1) then
		log:Debug("7")
		self.lootWindow:SetRoll(lootlog.LootWindow.GREED, dice, _StripItemName(itemName), self.playerName)
		return
	end
	local dice, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_YOU_WON_NO_SPAM_DE)
	if (match1) then
		log:Debug("8")
		self.lootWindow:SetRoll(lootlog.LootWindow.DISENCHANT, dice, _StripItemName(itemName), self.playerName)
		return
	end
	local playerName, dice, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_WON_NO_SPAM_NEED)
	if (match1) then
		log:Debug("9")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, dice, _StripItemName(itemName), playerName)
		return
	end
	local playerName, dice, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_WON_NO_SPAM_GREED)
	if (match1) then
		log:Debug("10")
		self.lootWindow:SetRoll(lootlog.LootWindow.GREED, dice, _StripItemName(itemName), playerName)
		return
	end
	local playerName, dice, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_WON_NO_SPAM_DE)
	if (match1) then
		log:Debug("11")
		self.lootWindow:SetRoll(lootlog.LootWindow.DISENCHANT, dice, _StripItemName(itemName), playerName)
		return
	end

	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_ALL_PASSED)
	if (itemName) then
		log:Debug("12")
		self.lootWindow:SetWinner(_StripItemName(itemName), nil)
		return
	end

	local itemName = LibDeformat.Deformat(msg, LOOT_ROLL_YOU_WON)
	if (itemName) then
		log:Debug("13")
		self.lootWindow:SetWinner(_StripItemName(itemName), self.playerName)
		return
	end
	local playerName, itemName = LibDeformat.Deformat(msg, LOOT_ROLL_WON)
	if (playerName) then
		log:Debug("14")
		self.lootWindow:SetWinner(_StripItemName(itemName), playerName)
		return
	end

	-- "Need Roll - %d for %s by %s + Role Bonus";
	local dice, itemName, playerName = LibDeformat.Deformat(msg, LOOT_ROLL_ROLLED_NEED_ROLE_BONUS)
	if (dice) then
		log:Debug("14.1")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end

	local dice, itemName, playerName = LibDeformat.Deformat(msg, LOOT_ROLL_ROLLED_DE)
	if (dice) then
		log:Debug("15")
		self.lootWindow:SetRoll(lootlog.LootWindow.DISENCHANT, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end
	local dice, itemName, playerName = LibDeformat.Deformat(msg, LOOT_ROLL_ROLLED_GREED)
	if (dice) then
		log:Debug("16")
		self.lootWindow:SetRoll(lootlog.LootWindow.GREED, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end
	local dice, itemName, playerName = LibDeformat.Deformat(msg, LOOT_ROLL_ROLLED_NEED)
	if (dice) then
		log:Debug("17")
		self.lootWindow:SetRoll(lootlog.LootWindow.NEED, _StripDice(dice), _StripItemName(itemName), playerName)
		return
	end
	log:Debug("exit without match")
end

function lootlog.LootLog:ProfileUpdate(event)
	log:Debug("ProfileUpdate event [%s]", event)
	self.lootWindow:ProfileUpdate()
end

function lootlog.LootLog:Update()
	self.lootWindow:TestModeUpdate()
end


function lootlog.LootLog:ChatCommand(msg)
	log:Debug("msg [%s]", msg)
	if ("debug" == msg) then
		if (log:IsDebugging()) then
			print("Turning off debugging")
			log:SetDebugging(nil)
		else
			print("Turning on debugging")
			log:SetDebugging(true)
		end
	elseif ("history" == msg) then
		print("Printing history")
		log:PrintHistory()
	elseif ("clear" == msg) then
		print("Clearing history")
		log:ClearHistory()
	else
		InterfaceOptionsFrame_OpenToCategory(self.configFrame)
	end
end

function lootlog.LootLog:START_LOOT_ROLL(evt, rollId, rollTime)
	log:Debug("START_LOOT_ROLL rollId [%s] rollTime [%s]", rollId, rollTime)
	
	if (not _IsActive(self)) then
		return
	end
	
	
	local texture, name, count, quality = GetLootRollItemInfo(rollId)
	local itemLink = GetLootRollItemLink(rollId)
	if (name and itemLink and quality >= self.db.profile.minQuality) then
		log:Debug("name [%s] quality [%s]", name, quality)
		self.lootWindow:AddRoll(rollId, name, itemLink, quality)
	end
end
